#!/usr/bin/env lua5.3

require 'luarocks.loader'
require 'luacov'

local daemonize = require 'daemonparts.daemonize'
local process = require 'daemonparts.process'	 -- useful for testing

local socket = require 'cqueues.socket'
local unix = require 'unix'

local proc = require 'tests.lib.process'
local file = require 'tests.lib.file'

math.randomseed( unix.clock_gettime() * 100 )
local pid_file


require 'busted.runner'()
describe("Daemonization Module", function()

	before_each(function()
		local rand = tostring(math.floor(math.random() * 10000))
		pid_file = '/tmp/daemonparts_pid-' .. rand
	end)

	after_each(function()
		unix.unlink(pid_file)
	end)


	it("Can fork", function()

		local com_p, com_c = socket.pair()
		local child = process.fork_fcn( function()
			daemonize.detach()
			local res = daemonize.pidfile( pid_file )

			com_c:xwrite("done\n", 'n')
			com_c:flush()
			com_c:xread("*l", 'n', 10)

			if not res then
				os.exit(10)
			end
			os.exit(0)
		end )

		assert.is_not_equal( unix.getpid(), child.pid )
		assert.is_equal(0, child:wait())

		local l, err = com_p:xread("*l", 'n', 10)
		assert.is_nil(err)
		assert.is_equal("done", l)

		local f = assert(io.open(pid_file, "r"))
		local d_pid = tonumber(f:read("*l"))
		f:close()

		assert.is_number(d_pid)
		assert.is_true( proc.process_exists( d_pid ) )
		assert.is_not_equal( unix.getpid(), d_pid )
		assert.is_not_equal( child.pid, d_pid )
		com_p:xwrite("die\n", 'n')

		child.close()
		com_c:close()
		com_p:close()

	end)


	it("Sets up a heartbeat", function()

		local com_p, com_c = socket.pair()
		local child = process.fork_fcn( function()
			daemonize.detach(true)
			daemonize.pidfile( pid_file )
			daemonize.ready()

			com_c:xwrite("done\n", 'n')
			com_c:flush()
			com_c:xread("*l", 'n', 10)
			os.exit(0)
		end )

		assert.is_not_equal( unix.getpid(), child.pid )
		local ecode = child:wait()
		assert.is_equal(0, ecode)
		assert.is_false( proc.process_exists( child.pid ) )

		local l, err = com_p:xread("*l", 'n', 10)
		assert.is_nil(err)
		assert.is_equal("done", l)

		local f = assert(io.open(pid_file, "r"))
		local d_pid = tonumber(f:read("*l"))
		f:close()

		assert.is_true( proc.process_exists( d_pid ) )
		assert.is_not_equal( unix.getpid(), d_pid )
		assert.is_not_equal( child.pid, d_pid )
		com_p:xwrite("die\n", 'n')

		child.close()
		com_c:close()
		com_p:close()

	end)


	it("Heartbeat signals failure", function()

		local child = process.fork_fcn( function()
			daemonize.detach(true)
			daemonize.error("nope")
		end )

		assert.is_not_equal( unix.getpid(), child.pid )
		assert.is_equal('nope', child.stdout:read("*line"))
		local ecode = child:wait()
		assert.is_equal(15, ecode)
		assert.is_false( proc.process_exists( child.pid ) )

		assert.is_false(file.exists(pid_file))
		child.close()

	end)


	it("Forgetting to call ready() or error() eventually stops", function()

		local child = process.fork_fcn( function ()
			daemonize.detach(true)
			os.exit(0)
		end )

		local now = os.time()

		assert.is_not_equal( unix.getpid(), child.pid )
		assert.is_equal('Unexpected Process Exit', child.stdout:read("*line"))
		local ecode = child:wait()
		assert.is_true((os.time() - now) < 10)

		assert.is_equal(16, ecode)
		assert.is_false( proc.process_exists( child.pid ) )

		assert.is_false(file.exists(pid_file))
		child.close()

	end)


	it("Can manually write a pidfile", function()

		local pid = unix.getpid()
		unix.unlink(pid_file .. '-parent')

		assert.is_false(file.exists( pid_file .. '-parent' ))
		daemonize.pidfile( pid_file .. '-parent' )
		assert.is_true(file.exists( pid_file .. '-parent' ))

		local f = io.open( pid_file .. '-parent', 'r' )
		local found_pid = tonumber(f:read("*line"))
		f:close()

		assert.is_equal(pid, found_pid)
		unix.unlink(pid_file .. '-parent')

	end)


	it("Can wrap it up into one function", function()

		local com_p, com_c = socket.pair()
		local child = process.fork_fcn( function ()
			daemonize.go( function()
				daemonize.pidfile( pid_file )
			end )

			com_c:xwrite("done\n", 'n')
			com_c:flush()
			com_c:xread("*l", 'n', 10)
			os.exit(0)
		end )

		assert.is_not_equal( unix.getpid(), child.pid )
		local ecode = child:wait()
		assert.is_equal(0, ecode)
		assert.is_false( proc.process_exists( child.pid ) )

		local l, err = com_p:xread("*l", 'n', 10)
		assert.is_nil(err)
		assert.is_equal("done", l)

		assert.is_true( file.exists( pid_file ))
		local f = assert(io.open(pid_file, "r"))
		local d_pid = tonumber(f:read("*l"))
		f:close()
		assert.is_true( proc.process_exists( d_pid ) )
		com_p:xwrite("die\n", 'n')

		assert.is_not_equal( unix.getpid(), d_pid )
		assert.is_not_equal( child.pid, d_pid )
		child.close()
		com_c:close()
		com_p:close()

	end)


	it("Can wrap it up into one function - and handle error", function()

		local child = process.fork_fcn( function ()
			daemonize.go( function()
				error("nope")
			end )
		end )

		assert.is_not_equal( unix.getpid(), child.pid )
		local ecode = child:wait()
		assert.is_equal(15, ecode)
		assert.is_false( proc.process_exists( child.pid ) )

	end)

end)
