#!/usr/bin/env lua5.3

require 'luarocks.loader'
require 'luacov'

local unix = require 'unix'
local file = require 'tests.lib.file'

local testfile = '/tmp/daemonparts-test-log-' .. tostring(unix.getpid())

local output = require 'daemonparts.output'


require 'busted.runner'()
describe("Output Module", function()

	setup(function()
		output.switch( 'file', testfile )
	end)

	before_each(function()
		unix.unlink(testfile)
		output.reset()
		assert.is_true(file.exists(testfile))
	end)

	teardown(function()
		unix.unlink(testfile)
	end)

	it("Removes only debug by default", function()

		output.filter('info')

		output.info("normal")
		output.notice("note")
		output.debug("debug")
		output.warning("warn")
		output.error("error")

		local reg = '(%d+)%-(%d+)%-(%d+) (%d+)%:(%d+)%:(%d+)'
		local expected_lines = {
			reg .. ' normal',
			reg .. ' NOTICE: note',
			reg .. ' WARNING: warn',
			reg .. ' ERROR: error'
		}

		local f = io.open(testfile, 'r')
		for i, expected in ipairs(expected_lines) do	-- luacheck: ignore 213
			local line = f:read("*line")
			assert.is_match(expected, line)
		end

		f:close()

	end)


	it("Can lower the filter threshold", function()

		output.filter('debug')

		output.info("info")
		output.notice("notice this")
		output.debug("debug")
		output.error("error")


		local reg = '(%d+)%-(%d+)%-(%d+) (%d+)%:(%d+)%:(%d+)'
		local expected_lines = {
			reg .. ' info',
			reg .. ' NOTICE: notice this',
			reg .. ' DEBUG: debug',
			reg .. ' ERROR: error'
		}

		local f = io.open(testfile, 'r')
		for i, expected in ipairs(expected_lines) do	-- luacheck: ignore 213
			local line = f:read("*line")
			assert.is_match(expected, line)
		end

		f:close()

	end)


	it("Can raise the filter threshold", function()

		output.filter('error')

		output.info("info")
		output.debug("debug")
		output.info("info")
		output.info("info")
		output.info("info")
		output.warning("warning")
		output.info("info")
		output.error("error")

		local reg = '(%d+)%-(%d+)%-(%d+) (%d+)%:(%d+)%:(%d+)'
		local expected_lines = {
			reg .. ' ERROR: error'
		}

		local f = io.open(testfile, 'r')
		for i, expected in ipairs(expected_lines) do	-- luacheck: ignore 213
			local line = f:read("*line")
			assert.is_match(expected, line)
		end

		f:close()

	end)


	it("Can rotate the logfile with reset()", function()

		output.filter("info")
		local reg = '(%d+)%-(%d+)%-(%d+) (%d+)%:(%d+)%:(%d+)'

		output.info("info")
		local f = io.open(testfile, 'r')
		assert.is_match(reg .. ' info', f:read("*line"))
		f:close()

		unix.unlink(testfile)
		output.reset()

		output.info("info2")
		local f2 = io.open(testfile, 'r')
		assert.is_match(reg .. ' info2', f2:read("*line"))
		f2:close()

	end)


	it("Calling reset() doesn't overwrite", function()

		output.filter("info")
		local reg = '(%d+)%-(%d+)%-(%d+) (%d+)%:(%d+)%:(%d+)'

		output.info("info")
		local f = io.open(testfile, 'r')
		assert.is_match(reg .. ' info', f:read("*line"))
		f:close()

		output.reset()

		output.info("info2")
		local f2 = io.open(testfile, 'r')
		assert.is_match(reg .. ' info', f2:read("*line"))
		assert.is_match(reg .. ' info2', f2:read("*line"))
		f2:close()

	end)


	it("Rejects unknown filter values", function()

		-- sane - no error
		output.filter('debug')
		output.filter('notice')
		output.filter('info')
		output.filter('error')

		-- not sane - anything else
		assert.is_error(function()
			output.filter()
		end)

		assert.is_error(function()
			output.filter(353)
		end)

		assert.is_error(function()
			output.filter(function() end)
		end)

		assert.is_error(function()
			output.filter({})
		end)

		assert.is_error(function()
			output.filter(false)
		end)

		assert.is_error(function()
			output.filter("")
		end)

		assert.is_error(function()
			output.filter("some other string")
		end)

	end)

end)
