#!/usr/bin/env lua5.3

require 'luarocks.loader'
require 'luacov'

local proc_lib = require 'tests.lib.process'
local process = require 'daemonparts.process'
local unix = require 'unix'

math.randomseed( os.time() )



require 'busted.runner'()
describe("Child Process Module", function()

	it("Fork rejects unknown arguments", function()

		assert.is_error(function()
			process.fork()
		end)

		assert.is_error(function()
			process.fork("thingy")
		end)

		assert.is_error(function()
			process.fork(34534534)
		end)

		assert.is_error(function()
			process.fork(true)
		end)

		assert.is_error(function()
			process.fork(function() end)
		end)

	end)


	it("Fork_fcn rejects unknown arguments", function()
		assert.is_error(function()
			process.fork_fcn()
		end)

		assert.is_error(function()
			process.fork_fcn("thing")
		end)

		assert.is_error(function()
			process.fork_fcn(3523523)
		end)

		assert.is_error(function()
			process.fork_fcn(true)
		end)

		assert.is_error(function()
			process.fork_fcn({})
		end)

	end)


	it("Starts a process", function()

		local pid = unix.getpid()

		local child = process.fork_fcn(function()
			unix.sleep(0.3)
		end)


		assert.is_table(child)
		assert.is_number(child.pid)
		assert.is_true(proc_lib.process_exists( child.pid ))
		assert.is_not_equal(pid, child.pid)

		assert.is_equal(0, child.wait())
		assert.is_false(proc_lib.process_exists( child.pid ))

		-- prove we're the same process and not the child
		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Accepts a table", function()

		local pid = unix.getpid()

		local child = process.fork {
			entry = function()
				unix.sleep(0.3)
			end
		}


		assert.is_table(child)
		assert.is_number(child.pid)
		assert.is_true(proc_lib.process_exists( child.pid ))
		assert.is_not_equal(pid, child.pid)

		assert.is_equal(0, child.wait())
		assert.is_false(proc_lib.process_exists( child.pid ))

		-- prove we're the same process and not the child
		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Gets the correct error code", function()

		local pid = unix.getpid()
		local expected = math.floor(math.random() * 100 )
		expected = expected + 5 -- ensure it's not zero

		local child = process.fork_fcn(function()
			unix.sleep(0.3)
			os.exit( expected )
		end)


		assert.is_table(child)
		assert.is_number(child.pid)
		assert.is_true(proc_lib.process_exists( child.pid ))
		assert.is_not_equal(pid, child.pid)

		assert.is_equal(expected, child.wait())
		assert.is_false(proc_lib.process_exists( child.pid ))

		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Survives an error", function()

		local pid = unix.getpid()

		process.fork_fcn(function()
			error("Whoops")
		end)

		assert.is_equal(pid, unix.getpid())

	end)


	it("Redirects stdout/stdin", function()

		local pid = unix.getpid()

		local child = process.fork_fcn(function()
			print("start")
			print(io.stdin:read())
			print("end\n")
		end)

		assert.is_table(child)
		assert.is_not_nil(child.stdin)
		assert.is_not_nil(child.stdout)

		assert.is_equal("start", child.stdout:read("*l"))
		local line = "test-" .. tostring(math.random())
		child.stdin:write(line .."\n")
		child.stdin:flush()
		assert.is_equal(line, child.stdout:read("*l"))
		assert.is_equal("end", child.stdout:read("*l"))

		assert.is_equal(0, child:wait())
		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Redirects stderr", function()

		local pid = unix.getpid()
		local line = "test-" .. tostring(math.random())

		local child = process.fork_fcn(function()
			io.stderr:write( line .. "\n" )
		end)

		assert.is_table(child)
		assert.is_not_nil(child.stderr)
		assert.is_equal(line, child.stderr:read("*l"))

		assert.is_equal(0, child:wait())
		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Merges stderr into stdout", function()

		local pid = unix.getpid()
		local line = "test-" .. tostring(math.random())
		local line2 = "test-" .. tostring(math.random())

		local child = process.fork {
			merge_stderr = true,
			entry = function()
				io.stderr:write( line .. "\n" )
				io.stderr:flush()
				io.stdout:write( line2 .. "\n" )
				io.stderr:flush()
			end
		}

		assert.is_table(child)
		assert.is_nil(child.stderr)
		assert.is_equal(line, child.stdout:read("*l"))
		assert.is_equal(line2, child.stdout:read("*l"))

		assert.is_equal(0, child:wait())
		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Connects stdin to /dev/null", function()

		local pid = unix.getpid()
		local line = "test-" .. tostring(math.random())
		local line2 = "test-" .. tostring(math.random())

		local child = process.fork {
			stdin_null = true,
			entry = function()
				io.stderr:write( line .. "\n" )
				io.stderr:flush()
				io.stdout:write( line2 .. "\n" )
				io.stderr:flush()
			end
		}

		assert.is_table(child)
		assert.is_nil(child.stdin)
		assert.is_not_nil(child.stdout)
		assert.is_not_nil(child.stderr)
		assert.is_equal(line, child.stderr:read("*l"))
		assert.is_equal(line2, child.stdout:read("*l"))

		assert.is_equal(0, child:wait())
		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Connects stdout to /dev/null", function()

		local pid = unix.getpid()
		local line = "test-" .. tostring(math.random())
		local line2 = "test-" .. tostring(math.random())

		local child = process.fork {
			stdout_null = true,
			entry = function()
				io.stderr:write( line .. "\n" )
				io.stderr:flush()
				io.stdout:write( line2 .. "\n" )
				io.stderr:flush()
			end
		}

		assert.is_table(child)
		assert.is_nil(child.stdout)
		assert.is_not_nil(child.stdin)
		assert.is_not_nil(child.stderr)
		assert.is_equal(line, child.stderr:read("*l"))
		assert.is_nil(child.stderr:read("*l"))

		assert.is_equal(0, child:wait())
		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Connects stderr to /dev/null", function()

		local pid = unix.getpid()
		local line = "test-" .. tostring(math.random())
		local line2 = "test-" .. tostring(math.random())

		local child = process.fork {
			stderr_null = true,
			entry = function()
				io.stderr:write( line .. "\n" )
				io.stderr:flush()
				io.stdout:write( line2 .. "\n" )
				io.stderr:flush()
			end
		}

		assert.is_table(child)
		assert.is_not_nil(child.stdout)
		assert.is_not_nil(child.stdin)
		assert.is_nil(child.stderr)
		assert.is_equal(line2, child.stdout:read("*l"))
		assert.is_nil(child.stdout:read("*l"))

		assert.is_equal(0, child:wait())
		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Survives combining merge_stderr and stderr_null", function()

		local pid = unix.getpid()
		local line = "test-" .. tostring(math.random())
		local line2 = "test-" .. tostring(math.random())

		local child = process.fork {
			stderr_null = true,
			merge_stderr = true,
			entry = function()
				io.stderr:write( line .. "\n" )
				io.stderr:flush()
				io.stdout:write( line2 .. "\n" )
				io.stderr:flush()
			end
		}

		assert.is_table(child)
		assert.is_not_nil(child.stdout)
		assert.is_not_nil(child.stdin)
		assert.is_nil(child.stderr)
		assert.is_equal(line2, child.stdout:read("*l"))
		assert.is_nil(child.stdout:read("*l"))

		assert.is_equal(0, child:wait())
		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Survives combining merge_stderr and stdout_null", function()

		local pid = unix.getpid()
		local line = "test-" .. tostring(math.random())
		local line2 = "test-" .. tostring(math.random())

		local child = process.fork {
			stdout_null = true,
			merge_stderr = true,
			entry = function()
				io.stderr:write( line .. "\n" )
				io.stderr:flush()
				io.stdout:write( line2 .. "\n" )
				io.stderr:flush()
			end
		}

		assert.is_table(child)
		assert.is_not_nil(child.stdin)
		assert.is_nil(child.stdout)
		assert.is_nil(child.stderr)

		assert.is_equal(0, child:wait())
		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Forks-and-execs", function()

		local pid = unix.getpid()

		local child = process.exec_cmd("/usr/bin/env lua -v")

		assert.is_table(child)
		assert.is_number(child.pid)
		assert.is_not_equal(pid, child.pid)

		local line = child.stdout:read("*line")
		assert.is_match("^Lua ", line)

		assert.is_equal(0, child.wait())
		assert.is_false(proc_lib.process_exists( child.pid ))

		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Takes arguments as an array", function()

		local pid = unix.getpid()

		local child = process.exec_cmd("/usr/bin/env", "lua", "-v")

		assert.is_table(child)
		assert.is_number(child.pid)
		assert.is_not_equal(pid, child.pid)

		local line = child.stdout:read("*line")
		assert.is_match("^Lua ", line)

		assert.is_equal(0, child.wait())
		assert.is_false(proc_lib.process_exists( child.pid ))

		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Allows numeric arguments", function()

		local pid = unix.getpid()

		local child = process.exec_cmd("/bin/echo", "35")

		assert.is_table(child)
		assert.is_number(child.pid)
		assert.is_not_equal(pid, child.pid)

		local line = child.stdout:read("*line")
		assert.is_match("35", line)

		assert.is_equal(0, child.wait())
		assert.is_false(proc_lib.process_exists( child.pid ))

		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Forks, via table args", function()

		local pid = unix.getpid()

		local child = process.exec {
			command = "/usr/bin/env lua -v"
		}

		assert.is_table(child)
		assert.is_number(child.pid)
		assert.is_not_equal(pid, child.pid)

		local line = child.stdout:read("*line")
		assert.is_match("^Lua ", line)

		assert.is_equal(0, child.wait())
		assert.is_false(proc_lib.process_exists( child.pid ))

		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Execs, via split table args", function()

		local pid = unix.getpid()

		local child = process.exec {
			command = "/usr/bin/env",
			argv = {"lua", "-v"}
		}

		assert.is_table(child)
		assert.is_number(child.pid)
		assert.is_not_equal(pid, child.pid)

		local line = child.stdout:read("*line")
		assert.is_match("^Lua ", line)

		assert.is_equal(0, child.wait())
		assert.is_false(proc_lib.process_exists( child.pid ))

		assert.is_equal(pid, unix.getpid())
		child.close()

	end)


	it("Exec_cmd Rejects insane arguments", function()

		local pid = unix.getpid()

		assert.is_error(function()
			process.exec_cmd(nil, "lua", "-v")
		end)

		assert.is_error(function()
			process.exec_cmd(true, "lua", "-v")
		end)

		assert.is_error(function()
			process.exec_cmd(48574854, "lua", "-v")
		end)

		assert.is_error(function()
			process.exec_cmd({ 'things', 'things2' }, "lua", "-v")
		end)

		assert.is_error(function()
			process.exec_cmd(function() end, "lua", "-v")
		end)


		assert.is_error(function()
			process.exec_cmd("/usr/bin/env", "lua", true, "-v")
		end)

		assert.is_error(function()
			process.exec_cmd("/usr/bin/env", "lua", { 'things', 'others' })
		end)

		assert.is_error(function()
			process.exec_cmd("/usr/bin/env", "lua", function() end, "-v")
		end)


		assert.is_equal(pid, unix.getpid())
	end)


	it("Exec rejects unknown arguments", function()

		assert.is_error(function()
			process.exec()
		end)

		assert.is_error(function()
			process.exec("thingy")
		end)

		assert.is_error(function()
			process.exec(34534534)
		end)

		assert.is_error(function()
			process.exec(true)
		end)

		assert.is_error(function()
			process.exec(function() end)
		end)

	end)

end)
